@@ -127,7 +127,7 @@ def grp_ts_agg(df, grp_col, ts_col, freq_code, agg_fun, discrete=False, **kwargs
             if grp_col is None:
 
                 if discrete:
-                    df3 = discrete_resample(df1, freq_code, agg_fun, **kwargs)
+                    df3 = discrete_resample(df1, freq_code, agg_fun, True, **kwargs)
                 else:
                     df3 = df1.resample(freq_code, **kwargs).agg(agg_fun)
 
@@ -148,7 +148,7 @@ def grp_ts_agg(df, grp_col, ts_col, freq_code, agg_fun, discrete=False, **kwargs
                 grp_list = []
 
                 for i, r in grp1:
-                    s6 = discrete_resample(r[val_cols], freq_code, agg_fun, **kwargs)
+                    s6 = discrete_resample(r[val_cols], freq_code, agg_fun, True, **kwargs)
                     s6[grp_col] = i
 
                     grp_list.append(s6)
