@@ -19,6 +19,7 @@ from tethys_utils.misc import make_run_date_key, grp_ts_agg, write_pkl_zstd
 import geojson
 from shapely.geometry import shape, mapping, Point, box
 from shapely import wkb
+from tethysts import Tethys
 
 ############################################
 ### Parameters
@@ -75,6 +76,7 @@ def data_integrety_checks(data, parameter, ds_metadata, attrs, encoding):
         raise ValueError('spatial_distribution should be either sparse or grid.')
 
     ts_essential_list = [parameter]
+
     ts_no_attrs_list = ['modified_date', 'station_id', 'lat', 'lon', 'name', 'altitude', 'ref', 'virtual_station', 'geometry']
 
     for c in ts_index_list:
@@ -94,8 +96,15 @@ def data_integrety_checks(data, parameter, ds_metadata, attrs, encoding):
     if isinstance(encoding, dict):
         for col in vars1:
             if not col in ts_no_attrs_list:
-                if not col in encoding:
-                    raise ValueError(col + ' must be in the encoding dict')
+                if col in ancillary_variables:
+                    if not col in encoding:
+                        raise ValueError(col + ' must be in the encoding dict')
+
+    # if isinstance(encoding, dict):
+    #     for col in vars1:
+    #         if not col in ts_no_attrs_list:
+    #             if not col in encoding:
+    #                 raise ValueError(col + ' must be in the encoding dict')
 
     ## Geometry and metadata checks
     if 'geometry' in data_index:
@@ -181,49 +190,49 @@ def package_xarray(data, parameter, attrs, encoding, run_date=None, compression=
 
     ## Create extent and staion_id if grouping is blocks
     if grouping == 'blocks':
-        if sd == 'grid':
-            res = np.mean(np.diff(np.unique(data1['lon'])))
-            min_x = (data1['lon'].min() - (res*0.5)).round(7)
-            max_x = (data1['lon'].max() + (res*0.5)).round(7)
-            min_y = (data1['lat'].min() - (res*0.5)).round(7)
-            max_y = (data1['lat'].max() + (res*0.5)).round(7)
-        else:
-            geos1 = [wkb.loads(g, True) for g in data1['geometry'].values]
-            # TODO: Finish the lines and polygon extent creation
+        if 'station_id' not in data1:
+            if sd == 'grid':
+                res = np.mean(np.diff(np.unique(data1['lon'])))
+                min_x = (data1['lon'].min() - (res*0.5)).round(7)
+                max_x = (data1['lon'].max() + (res*0.5)).round(7)
+                min_y = (data1['lat'].min() - (res*0.5)).round(7)
+                max_y = (data1['lat'].max() + (res*0.5)).round(7)
+            else:
+                raise NotImplementedError('Need to implement lines and polygons blocks extents.')
+                geos1 = [wkb.loads(g, True) for g in data1['geometry'].values]
+                # TODO: Finish the lines and polygon extent creation
 
-        extent = box(min_x, min_y, max_x, max_y)
-        extent_geo = mapping(extent)
-        stn_id = assign_station_id(extent_geo)
-        extent_hex = extent.wkb_hex
-        data1 = data1.assign_coords([extent_hex])
-        data1 = data1.assign({'station_id': (('extent'), [stn_id])})
+            extent = box(min_x, min_y, max_x, max_y)
+            extent_geo = mapping(extent)
+            stn_id = assign_station_id(extent_geo)
+            extent_hex = extent.wkb_hex
+            data1 = data1.assign_coords({'extent': [extent_hex]})
+            data1 = data1.assign({'station_id': (('extent'), [stn_id])})
 
     ## Assign Attributes
-    attrs1 = {'station_id': {'cf_role': "timeseries_id"}, 'lat': {'standard_name': "latitude", 'units': "degrees_north"}, 'lon': {'standard_name': "longitude", 'units': "degrees_east"}, 'altitude': {'standard_name': 'surface_altitude', 'long_name': 'height above the geoid to the lower boundary of the atmosphere', 'units': 'm'}, 'geometry': {'long_name': 'The hexadecimal encoding of the Well-Known Binary (WKB) geometry', 'crs_EPSG': 4326}, 'extent': {'long_name': 'The hexadecimal encoding of the Well-Known Binary (WKB) station extent', 'crs_EPSG': 4326}}
+    attrs1 = {'station_id': {'cf_role': "timeseries_id"}, 'lat': {'standard_name': "latitude", 'units': "degrees_north"}, 'lon': {'standard_name': "longitude", 'units': "degrees_east"}, 'altitude': {'standard_name': 'surface_altitude', 'long_name': 'height above the geoid to the lower boundary of the atmosphere', 'units': 'm'}, 'geometry': {'long_name': 'The hexadecimal encoding of the Well-Known Binary (WKB) geometry', 'crs_EPSG': 4326}, 'extent': {'long_name': 'The hexadecimal encoding of the Well-Known Binary (WKB) station extent', 'crs_EPSG': 4326}, 'height': {'standard_name': 'height', 'long_name': 'vertical distance above the surface', 'units': 'm', 'positive': 'up'}, 'time': {'standard_name': 'time', 'long_name': 'start_time'}}
 
     if 'name' in vars1:
         attrs1.update({'name': {'long_name': 'station name'}})
     if 'ref' in vars1:
         attrs1.update({'ref': {'long_name': 'station reference id given by the owner'}})
 
+    if 'modified_date' in vars1:
+        attrs1.update({'modified_date': {'long_name': 'last modified date'}})
+
     if isinstance(attrs, dict):
-        attrs1.update(copy.deepcopy(attrs))
+        for k, v in attrs.items():
+            attrs1[k] = v
 
     if 'cf_standard_name' in attrs1[parameter]:
         attrs1[parameter]['standard_name'] = attrs1[parameter].pop('cf_standard_name')
-    attrs1.update({'height': {'standard_name': 'height', 'long_name': 'vertical distance above the surface', 'units': 'm', 'positive': 'up'}, 'time': {'standard_name': 'time', 'long_name': 'start_time'}})
-    if 'modified_date' in vars1:
-        attrs1.update({'modified_date': {'long_name': 'last modified date'}})
 
     if isinstance(ancillary_variables, list):
         if len(ancillary_variables) > 0:
             attrs1[parameter].update({'ancillary_variables': ' '.join(ancillary_variables)})
 
     ## Assign encodings
-    encoding1 = {'lon': {'dtype': 'int32', '_FillValue': -999999, 'scale_factor': 0.0000001}, 'lat': {'dtype': 'int32', '_FillValue': -999999, 'scale_factor': 0.0000001}, 'altitude': {'dtype': 'int32', '_FillValue': -9999, 'scale_factor': 0.001}}
-
-    if isinstance(encoding, dict):
-        encoding1.update(copy.deepcopy(encoding))
+    encoding1 = {'lon': {'dtype': 'int32', '_FillValue': -999999, 'scale_factor': 0.0000001}, 'lat': {'dtype': 'int32', '_FillValue': -999999, 'scale_factor': 0.0000001}, 'altitude': {'dtype': 'int32', '_FillValue': -9999, 'scale_factor': 0.001}, 'time': {'_FillValue': -99999999, 'units': "days since 1970-01-01 00:00:00"}}
 
     data1['height'] = pd.to_numeric(data1['height'].values, downcast='integer')
 
@@ -234,10 +243,14 @@ def package_xarray(data, parameter, attrs, encoding, run_date=None, compression=
     else:
         raise TypeError('height should be either an int or a float')
 
-    encoding1.update({'time': {'_FillValue': -99999999, 'units': "days since 1970-01-01 00:00:00"}, 'height': height_enc})
+    encoding1.update({'height': height_enc})
     if 'modified_date' in vars1:
         encoding1.update({'modified_date': {'_FillValue': -99999999, 'units': "days since 1970-01-01 00:00:00"}})
 
+    if isinstance(encoding, dict):
+        for k, v in encoding.items():
+            encoding1[k] = v
+
     ## Add attributes and encodings
     for e, val in encoding1.items():
         if e in data1:
@@ -528,12 +541,23 @@ def compare_xrs(old_xr, new_xr, add_old=False):
 
     ## Reduce the dimensions for the comparison for compatibility
     new1_s = new_xr[vars2].squeeze()
-    old1_s = old_xr[vars2].squeeze()
     on = list(new1_s.dims)
 
-    if not on == list(old1_s.dims):
+    # Fix for when there is no dimension > 1
+    if len(on) == 0:
+        new1_s = new1_s.expand_dims('time')
+        on = ['time']
+
+    old1_s = old_xr[vars2].squeeze()
+    old_on = list(old1_s.dims)
+    if len(old_on) == 0:
+        old1_s = old1_s.expand_dims('time')
+        old_on = ['time']
+
+    if not on == old_on:
         raise ValueError('Dimensions are not the same between the datasets')
 
+    ## Assign variables
     keep_vars = on + vars2
 
     new_all_vars = list(new1_s.variables)
@@ -576,7 +600,8 @@ def compare_xrs(old_xr, new_xr, add_old=False):
 
         for v in vars1:
             if v not in vars2:
-                comp2[v] = new_xr[v].copy()
+                if v not in on:
+                    comp2[v] = new_xr[v].copy()
                 comp2[v].attrs = new_xr[v].attrs.copy()
                 comp2[v].encoding = new_xr[v].encoding.copy()
 
@@ -691,6 +716,93 @@ def assign_station_ids_df(stns_df, extent=False, precision=5):
     return stn_ids
 
 
+def process_stations_df(stns, dataset_id=None, remote=None):
+    """
+
+    """
+    ## Get existing stations from tethys
+    if isinstance(dataset_id, str) and isinstance(remote, dict):
+        try:
+            tethys = Tethys([remote])
+            remote_stns = tethys.get_stations(dataset_id)
+            remote_stns1 = pd.DataFrame(remote_stns)[['station_id', 'ref']]
+
+            ## Assign station_ids
+            stns2 = pd.merge(stns, remote_stns1, on='ref', how='left')
+        except:
+            stns2 = stns.copy()
+            stns2['station_id'] = np.nan
+    else:
+        stns2 = stns.copy()
+        stns2['station_id'] = np.nan
+
+    stns2.loc[stns2['station_id'].isnull(), 'station_id'] = assign_station_ids_df(stns2[stns2['station_id'].isnull()])
+    stns2 = stns2.drop_duplicates('station_id').copy()
+
+    ## Assign geometries
+    stns2['geometry'] = create_geometry_df(stns2, to_wkb_hex=True, precision=6)
+
+    ## Final station processing
+    stns3 = stns2.drop(['lat', 'lon'], axis=1)
+
+    return stns3
+
+
+def stations_dict_to_df(stns):
+    """
+
+    """
+    s1 = copy.deepcopy(stns)
+    s0 = [s.pop('stats') for s in s1]
+    s0 = [s.pop('virtual_station') for s in s1 if 'virtual_station' in s]
+    s0 = [s.pop('modified_date') for s in s1 if 'modified_date' in s]
+    s0 = [s.pop('dataset_id') for s in s1]
+
+    ## Process attrs
+    attrs = {}
+    for s in s1:
+        if 'properties' in s:
+            if s['properties']:
+                for pk, pv in s['properties'].items():
+                    attrs.update({pk: pv['attrs']})
+                    if isinstance(pv['data'], list):
+                        s.update({pk: pv['data'][0]})
+                    else:
+                        s.update({pk: pv['data']})
+            s.pop('properties')
+
+    ## Convert to df
+    s2 = pd.DataFrame(s1)
+
+    ## Process geometry
+    s2['geometry'] = create_geometry_df(s2, to_wkb_hex=True, precision=6)
+
+    ## Return
+    return s2, attrs
+
+
+def prepare_new_xr(ts_data, stn_data, mod_date=True):
+    """
+
+    """
+    stn = stn_data.to_frame().T.set_index('geometry').to_xarray()
+
+    obs2 = ts_data.copy()
+
+    obs2['geometry'] = stn['geometry'].values[0]
+
+    obs2.set_index(['geometry', 'height', 'time'], inplace=True)
+
+    if mod_date:
+        mod_date = pd.Timestamp.today(tz='utc').round('s').tz_localize(None)
+        obs2['modified_date'] = mod_date
+
+    obs3 = obs2.to_xarray()
+    obs4 = xr.combine_by_coords([obs3, stn], data_vars='minimal')
+
+    return obs4
+
+
 def get_new_stats(data):
     """
 
@@ -713,53 +825,6 @@ def get_new_stats(data):
     return stats1
 
 
-# def process_stations_df(stns_df, precision=6):
-#     """
-#     For processing the initial stations prior to assigning the altitude.
-#     """
-#     stns1 = stns_df.copy()
-#     stns1['coords'] = stns1.apply(lambda x: [x.lon, x.lat], axis=1)
-
-#     stns1['geometry'] = stns1.apply(lambda x: geojson.Point(x.coords, True, precision=precision), axis=1)
-#     stns1['station_id'] = stns1.apply(lambda x: assign_station_id(x.geometry), axis=1)
-
-#     stns2 = stns1.drop_duplicates('station_id').drop(['coords', 'lat', 'lon'], axis=1).copy()
-
-#     return stns2
-
-
-# def process_station_base(coords=None, station_id=None, geometry=None, ref=None, name=None, osm_id=None, altitude=None, properties=None, return_dict=True):
-#     """
-
-#     """
-#     if isinstance(coords, list):
-#         ## Create geometry and station_id
-#         geometry = geojson.Point(coords, True, precision=5)
-#         station_id = assign_station_id(geometry)
-#     elif (not isinstance(station_id, str)) & (not isinstance(geometry, dict)):
-#         raise ValueError('coords or station_id must be assigned')
-
-#     ## Put into data model
-#     stn_m = StationBase(station_id=station_id, geometry=geometry, ref=ref, name=name, osm_id=osm_id, altitude=altitude, properties=properties)
-
-#     if return_dict:
-#         stn_m = orjson.loads(stn_m.json(exclude_none=True))
-
-#     return stn_m
-
-
-# def process_stations_base(stns_list):
-#     """
-
-#     """
-#     stns_dict = {}
-#     for s in stns_list:
-#         stn_m = process_station_base(**s)
-#         stns_dict[stn_m['station_id']] = stn_m
-
-#     return stns_dict
-
-
 def get_station_data_from_xr(data):
     """
     Parameters
@@ -799,7 +864,7 @@ def get_station_data_from_xr(data):
     # if not 'virtual_station' in stn_data1:
     #     stn_data1['virtual_station'] = False
 
-    props = {s: {'data': data[s].to_dict()['data'], 'attrs': data[s].to_dict()['attrs']} for s in stn_vars if s not in stn_fields}
+    props = {s: {'data': data[s].to_dict()['data'][0], 'attrs': data[s].to_dict()['attrs']} for s in stn_vars if s not in stn_fields}
     if props:
         stn_data1['properties'] = props
 
@@ -837,12 +902,6 @@ def prepare_results(data_dict, dataset_list, results_data, run_date_key, sum_clo
     """
 
     """
-    # if :
-    #     mod_date = pd.Timestamp(mod_date)
-    #     ancillary_variables = ['modified_date']
-    # else:
-    #     ancillary_variables = []
-
     tz_str = 'Etc/GMT{0:+}'
 
     parameter = dataset_list[0]['parameter']
@@ -864,11 +923,6 @@ def prepare_results(data_dict, dataset_list, results_data, run_date_key, sum_clo
 
     main_vars = [parameter] + ancillary_variables
 
-    # if not isinstance(results_data.index, pd.MultiIndex):
-    #     raise TypeError('The data_df index must be a MultiIndex.')
-
-    # data_index = list(results_data.index.names)
-
     if not 'time' in data_index:
         raise ValueError('time must be in the data_df index.')
 
@@ -932,9 +986,7 @@ def prepare_results(data_dict, dataset_list, results_data, run_date_key, sum_clo
             for av in ancillary_variables:
                 if qual_col == av:
                     ts_data2[qual_col] = pd.to_numeric(ts_data2[qual_col], errors='coerce', downcast='integer')
-                    qual_cols = data_index.copy()
-                    qual_cols.extend([qual_col])
-                    qual1 = grp_ts_agg(ts_data2[qual_cols].reset_index(), other_index, 'time', freq_code, 'min')
+                    qual1 = grp_ts_agg(ts_data2[qual_col].reset_index(), other_index, 'time', freq_code, 'min')
                     av_list.append(qual1)
                 else:
                     av1 = grp_ts_agg(ts_data2[av].reset_index(), other_index, 'time', freq_code, 'max')
@@ -951,13 +1003,18 @@ def prepare_results(data_dict, dataset_list, results_data, run_date_key, sum_clo
         if (not freq_code in ['T', 'H', '1H']) and (not utc_offset == '0H'):
             data2['time'] = data2['time'].to_index().tz_localize(tz1).tz_convert('utc').tz_localize(None)
 
-        ## Package up the netcdf object
-        new1 = package_xarray(data2, parameter, attrs1, encoding1, run_date=run_date_key, compression='zstd')
+        ## Check if the entire record is nan
+        n_data = int(data2[parameter].notnull().sum())
 
-        ## Update the data_dict
-        ds_id = ds_mapping['dataset_id']
+        if n_data > 0:
 
-        data_dict[ds_id].append(new1)
+            ## Package up the netcdf object
+            new1 = package_xarray(data2, parameter, attrs1, encoding1, run_date=run_date_key, compression='zstd')
+
+            ## Update the data_dict
+            ds_id = ds_mapping['dataset_id']
+
+            data_dict[ds_id].append(new1)
 
 
 
