@@ -11,10 +11,11 @@ import pandas as pd
 import os
 import glob
 from tethys_utils.processing import write_pkl_zstd, process_datasets, prepare_results, assign_station_id, make_run_date_key
-from tethys_utils.s3 import process_run_date, update_results_s3
+from tethys_utils.s3 import process_run_date, update_results_s3, put_remote_dataset, put_remote_agg_stations, put_remote_agg_datasets, s3_connection
 from shapely.geometry import shape, mapping, Point, box
 import copy
 import rasterio
+import concurrent.futures
 
 
 ###########################################
@@ -294,8 +295,7 @@ def _process_grid_blocks(block_list, band_parameter_dict, time, feature, method,
             ## Package the results
             prepare_results(data_dict, ds_list, block2, run_date_key)
 
-    return data_dict
-
+    return data_dict, dataset_list
 
 
 ############################################
@@ -350,7 +350,7 @@ class Raster(object):
         return size_dict
 
 
-    def process_save_results(self, remote, x_size, y_size, band_parameter_dict, time, feature, method, product_code, owner, aggregation_statistic, units, data_license, attribution, encoding, description, frequency_interval='T', utc_offset='0H', run_date=None, height=0, threads=30, public_url=None):
+    def process_save_results(self, remote, x_size, y_size, band_parameter_dict, time, feature, method, product_code, owner, aggregation_statistic, units, data_license, attribution, encoding, description, frequency_interval='T', utc_offset='0H', run_date=None, height=0, public_url=None, threads=30):
         """
 
         """
@@ -360,14 +360,44 @@ class Raster(object):
         ## Iterate through the images
         images = list(self.images.keys())
         images.sort()
+        # datasets = []
         for tif in images:
             print(tif)
 
             xr1 = xr.open_rasterio(tif)
             block_list = _split_grid(xr1, x_size=x_size, y_size=y_size, x_name=self.x_name, y_name=self.y_name)
-            data_dict = _process_grid_blocks(block_list, band_parameter_dict, time, feature, method, product_code, owner, aggregation_statistic, units, data_license, attribution, encoding, description, frequency_interval, utc_offset, self.x_name, self.y_name, run_date, height)
+            data_dict, dataset_list = _process_grid_blocks(block_list, band_parameter_dict, time, feature, method, product_code, owner, aggregation_statistic, units, data_license, attribution, encoding, description, frequency_interval, utc_offset, self.x_name, self.y_name, run_date, height)
             run_date_dict = {ds: run_date_key for ds in data_dict}
             update_results_s3(6, data_dict, run_date_dict, remote, threads, public_url)
+            # datasets.extend(dataset_list)
+
+
+        # def do_the_work(tif):
+        #     """
+
+        #     """
+        #     xr1 = xr.open_rasterio(tif)
+        #     block_list = _split_grid(xr1, x_size=x_size, y_size=y_size, x_name=self.x_name, y_name=self.y_name)
+        #     data_dict, dataset_list = _process_grid_blocks(block_list, band_parameter_dict, time, feature, method, product_code, owner, aggregation_statistic, units, data_license, attribution, encoding, description, frequency_interval, utc_offset, self.x_name, self.y_name, run_date, height)
+        #     run_date_dict = {ds: run_date_key for ds in data_dict}
+        #     update_results_s3(6, data_dict, run_date_dict, remote, threads, public_url)
+
+        #     return dataset_list
+
+
+        # with concurrent.futures.ProcessPoolExecutor(max_workers=2) as executor:
+        #     datasets = executor.map(do_the_work, images)
+
+        ## Update the datasets and station jsons
+        print('Aggregating dataset and station data.')
+        s3 = s3_connection(remote['connection_config'], 50)
+
+        for ds in dataset_list:
+            ds_new = put_remote_dataset(s3, remote['bucket'], ds)
+            ds_stations = put_remote_agg_stations(s3, remote['bucket'], ds['dataset_id'], 50)
+
+        # Aggregate all datasets for the bucket
+        ds_all = put_remote_agg_datasets(s3, remote['bucket'], 50)
 
         print('Processing and saving data has been successful!')
 
